import logging
import re

from pathlib import Path
from .constants import HTTP_NO_PORT_REGEX


BASE_DIR = Path(__file__).resolve().parent
logger = logging.getLogger(__name__)


class AbstractSpidCheck(object):
    def __init__(self, *args, **kwargs):
        self.results = []
        self.errors = []
        self.logger = logger
        self.error_counter = 0
        self.verify_ssl = kwargs.get("verify_ssl", False)
        self.category = ""

    def report_to_dict(self):
        res = {self.category: {self.__class__.__name__: self.results}}

        return res

    def is_ok(self, msg):
        if not self.error_counter:
            self.handle_result("info", f"{msg}")
            return True
        else:
            self.error_counter = 0
            return False

    def handle_result(
        self, level: str, title: str, description: str = "", traceback: str = None
    ):
        msg = f"{title}"
        getattr(self.logger, level, "debug")(msg)
        value = f"{description}" if not traceback else f"{description}: {traceback }"
        if level not in ("error", "debug", "critical", "warning"):
            # here report as json
            self.results.append(
                {
                    "result": "success",
                    "test": title,
                    # "value": value
                }
            )
        elif level in ("error", "critical"):
            self.handle_error(title, description, traceback)
        elif level == "warning":
            self.results.append(
                {
                    "result": "warning",
                    "test": title,
                    "value": value,
                }
            )

    def handle_error(self, error_message, description="", traceback: str = None):
        getattr(self.logger, "error")(error_message)
        self.error_counter += 1
        # here report as json
        value = f"{description}" if not traceback else f"{description}: {traceback }"
        data = {"result": "failure", "test": error_message, "value": value}
        self.errors.append(data)
        self.results.append(data)

    def _assert(
        self,
        check: bool,
        error_message: str,
        description="",
        traceback: str = None,
        level: str = "info",
    ):
        if not check and level == "info":
            self.handle_error(error_message, description, traceback)
        elif not check and level == "warning":
            self.handle_result(level, f"{error_message}", description, traceback)
        else:
            level = "info" if level in ("warning",) else level
            self.handle_result(level, f"{error_message}", description, traceback)

    def _assertTrue(
        self,
        check,
        error_message,
        description="",
        traceback: str = None,
        level: str = "info",
    ):
        self._assert(check, error_message, description, traceback, level)

    def _assertFalse(
        self,
        check,
        error_message,
        description="",
        traceback: str = None,
        level: str = "info",
    ):
        self._assert(not check, error_message, description, traceback, level)

    def _assertIsNotNone(
        self,
        check,
        error_message,
        description="",
        traceback: str = None,
        level: str = "info",
    ):
        self._assert(check, error_message, description, traceback, level)

    def _assertIn(
        self,
        first,
        second,
        error_message,
        description="",
        traceback: str = None,
        level: str = "info",
    ):
        self._assert((first in second), error_message, description, traceback, level)

    def _assertGreaterEqual(
        self,
        first,
        second,
        error_message,
        description="",
        traceback: str = None,
        level: str = "info",
    ):
        self._assert((first >= second), error_message, description, traceback, level)

    def _assertGreater(
        self,
        first,
        second,
        error_message,
        description="",
        traceback: str = None,
        level: str = "info",
    ):
        self._assert((first > second), error_message, description, traceback, level)

    def _assertEqual(
        self,
        first,
        second,
        error_message,
        description="",
        traceback: str = None,
        level: str = "info",
    ):
        self._assert((first == second), error_message, description, traceback, level)

    def _assertIsValidHttpsUrl(
        self,
        check,
        error_message,
        description="",
        traceback: str = None,
        level: str = "info",
    ):
        self._assert(
            re.match("https://", check if check else ""), description, traceback, level
        )

    def _assertHttpUrlWithoutPort(
        self,
        check,
        error_message,
        description="",
        traceback: str = None,
        level: str = "info",
    ):
        self._assert(
            re.match(HTTP_NO_PORT_REGEX, check if check else ""),
            description,
            traceback,
            level,
        )

    def _assertIsValidHttpUrl(
        self,
        check,
        error_message,
        description="",
        traceback: str = None,
        level: str = "info",
    ):
        self._assert(
            re.match("https?://", check if check else ""), description, traceback, level
        )

    # maybe useful .. one day ?!
    # idp_server = self.idp()
    # pysaml2 auth req object with signature check

    # SPID IdP not completely compliant to SAML2-core about Destination attribute
    # saml2.request:http://localhost:8088 not in ['http://localhost:54321/sso']
    # try:
    # req_obj = idp_server.parse_authn_request(self.authn_request_encoded,
    # BINDING_HTTP_POST)
    # except OtherError as e:
    # pass



from spid_sp_test.response import SpidSpResponse, SpidSpResponseCheck
from .settings import HTTP_STATUS_ERROR_CODES


def dynamic_acr(check: SpidSpResponseCheck, response_obj: SpidSpResponse, **kwargs):
    if check.get_acr() != check.response_attrs["AuthnContextClassRef"]:
        response_obj.conf["status_codes"] = HTTP_STATUS_ERROR_CODES
        try:
            level_sp = int(check.get_acr()[-1])
            level_idp = int(check.response_attrs["AuthnContextClassRef"][-1])
            if level_idp > level_sp:
                response_obj.conf["status_codes"] = [200]
        except Exception:
            pass



import os
import logging
import shutil

from . import BASE_DIR

logger = logging.getLogger(__name__)


from jinja2 import Environment, FileSystemLoader, select_autoescape


def render_html_report(
    data: dict,
    display_name: str,
    template_search_path: str = f"{BASE_DIR}/html",
    output_folder: str = "./html_report",
):
    loader = Environment(
        loader=FileSystemLoader(searchpath=template_search_path),
        autoescape=select_autoescape(["html"]),
    )

    template = loader.get_template("index.html")

    page_fname = f"{output_folder}/index.html"
    try:
        os.makedirs(output_folder, exist_ok=True)
        shutil.copytree(f"{template_search_path}/static", f"{output_folder}/static")
    except FileExistsError as e:
        logger.warning(e)

    f = open(page_fname, "w")
    f.write(template.render(report_data=data, display_name=display_name))
    f.close()



class SpidSpMetadataCheckPrivate(object):
    def test_Contacts_Priv(self):
        self.doc.xpath("//ContactPerson")

        ipacode = self.doc.xpath("//ContactPerson/Extensions/IPACode")
        self._assertFalse(
            ipacode,
            "The IPACode element MUST NOT be present",
            description=ipacode,
        )

        exts = self.doc.xpath("//ContactPerson/Extensions/CessionarioCommittente")
        self._assertTrue(
            (len(exts) == 1),
            ("The CessionarioCommittente element MUST be present"),
            description=exts,
        )

        if exts:
            exts[0]
            tise = self.doc.xpath(
                "//ContactPerson/Extensions/CessionarioCommittente/TerzoIntermediarioSoggettoEmittente"
            )
            if tise:
                tise = tise[0]
                self._assertTrue(
                    tise.text,
                    "If the TerzoIntermediarioSoggettoEmittente element if present it MUST have a value",
                    description=tise,
                )

        return self.is_ok(f"{self.__class__.__name__}.test_Contacts_Priv")



from glob import glob
from setuptools import setup

def readme():
    with open('README.md') as f:
        return f.read()

_src_folder = 'src'
_pkg_name = 'spid_sp_test'

setup(
    name=_pkg_name,
    version='0.9.5',
    description="SAML2 SPID Service Provider validation tool that can be run from the command line",
    long_description=readme(),
    long_description_content_type='text/markdown',
    classifiers=['Development Status :: 5 - Production/Stable',
                 'License :: OSI Approved :: European Union Public Licence 1.2 (EUPL 1.2)',
                 'Programming Language :: Python :: 3'],
    url='https://github.com/italia/spid-sp-test',
    author='Giuseppe De Marco',
    author_email='giuseppe.demarco@tamdigitale.governo.it',
    license='License :: OSI Approved :: European Union Public Licence 1.2 (EUPL 1.2)',
    scripts=['src/spid_sp_test/spid_sp_test'],
    packages=[f"{_pkg_name}"],
    package_dir={f"{_pkg_name}": f"{_src_folder}/{_pkg_name}"},

    package_data={f"{_pkg_name}": [i.replace(f'{_src_folder}/{_pkg_name}/', '')
                                   for i in glob(f'{_src_folder}/{_pkg_name}/**',
                                                 recursive=True)]
    },
    install_requires=[
                'pysaml2>=6.5.1',
                'xmlschema>=1.5.1',
                'requests>=2.25.1',
                'pyXMLSecurity>=0.21',
                'lxml>=4.6.2',
                'Jinja2>=2.11.3',
                # 'sslyze>=4.0.4', # todo
              ],
    )
